/////////////////////////////////////////////////////////////////////////////
// class CZoomView supports simple scrolling and scaling

class CZoomView : public CView
{
	DECLARE_DYNAMIC(CZoomView)

// Constructors
protected:
	CZoomView();

public:
#ifdef WIN32
	static AFX_DATA const SIZE sizeDefault;
#else
	static const SIZE AFXAPI_DATA sizeDefault;
#endif

		// used to specify default calculated page and line sizes
		
    enum ZoomAlignment { ZV_DEFAULT, ZV_TOPLEFT, ZV_BOTTOMLEFT, ZV_CENTER };
    enum { MINZOOM = 10, MAXZOOM = 1000 };

	// in logical units - call one of the following Set routines
	// compatibility to CScrollView
	void SetScrollSizes(int nMapMode, SIZE sizeTotal,
				const SIZE& sizePage = sizeDefault,
				const SIZE& sizeLine = sizeDefault );
	// three overloaded enhanced Versions to set the sizes
	// allows control of logical origin
	void SetDeviceScrollSizesRelative(int nMapMode, const CRect& rectDoc,
				int nPagePercent = 25, int nLinePercent = 5 );
	// supports arbitrary (ANISOTROPIC) mappings 
	void SetDeviceScrollSizesRelative(SIZE sizeDevice, const CRect& rectDoc,
				int nPagePercent = 25, int nLinePercent = 5 );
	// most compatible version to CScrollView
	void SetDeviceScrollSizesRelative(int nMapMode, SIZE sizeTotal,
		int nPagePercent, int nLinePercent );
	void SetAlignToBottom( BOOL bAlignToBottom ) { m_bAlignToBottom = bAlignToBottom; }
	void SetZoomAlign( ZoomAlignment align );

// Attributes
public:
	CPoint GetScrollPosition( ZoomAlignment Align = ZV_DEFAULT ) const; 
		// indicated position of scrolling
		
	CSize GetTotalSize() const { return m_totalLog; }   // logical size

	// for device units
	CPoint GetDeviceScrollPosition() const;
	void GetDeviceScrollSizes(int& nMapMode, SIZE& sizeTotal,
			SIZE& sizePage, SIZE& sizeLine) const;
	int GetZoomLevel()  const { return( m_nZoomLevel ); }
    BOOL GetAlignToBottom() const { return( m_bAlignToBottom ); }
    ZoomAlignment GetZoomAlign() { return( m_align );}

    
// Operations
public:
	void ScrollToPosition(POINT pt, ZoomAlignment Align = ZV_DEFAULT );    
		// set special position
		
	void FillOutsideRect(CDC* pDC, CBrush* pBrush);
	int SetZoomLevel( int nNewLevel, ZoomAlignment Align = ZV_DEFAULT );
	void ZoomToWindow();
	void ZoomToRectangle( CRect rectZoom, ZoomAlignment Align = ZV_DEFAULT );

// Implementation
protected:
	int m_nMapMode;
	CSize m_totalLog;         		// total size in logical units (no rounding)
	CRect m_rectLog;		  		// bounding rectangle of document in logical units
	CSize m_totalDev;         		// total size in device units, scaled
	CSize m_pageDev;          		// per page scroll size in device units
	CSize m_lineDev;          		// per line scroll size in device units
	CSize m_sizeDev;         		// total size in device units, unscaled for MM_ANISOTROPIC
	BOOL m_bCenter;          		// Center output if larger than total size
	BOOL m_bInsideUpdate;    		// internal state for OnSize callback
	void ScrollToDevicePosition(POINT ptDev); // explicit scrolling no checking
	int m_nZoomLevel;				// percentage of zooming
	BOOL m_bAlignToBottom;		    // align to bottom of window if it's bigger than necessary
	int m_nPagePercent;             // percentage of window to scroll
	int m_nLinePercent;
	BOOL m_bUseRelativeScrollSizes;
	ZoomAlignment m_align;
	// helper functions for zooming
	void SetMapMode( CDC* pDC ) const;
	void ScaleViewport( CDC* pDC ) const { pDC->ScaleViewportExt( m_nZoomLevel, 100, m_nZoomLevel, 100 ); }

protected:
	virtual void OnDraw(CDC* pDC) = 0;      // pass on pure virtual

	void UpdateBars(BOOL bSendRecalc = TRUE); // adjust scrollbars etc
	BOOL GetTrueClientSize(CSize& size, CSize& sizeSb);
		// size with no bars
	void GetScrollBarSizes(CSize& sizeSb);
	void GetScrollBarState(CSize sizeClient, CSize& needSb,
		CSize& sizeRange, CPoint& ptMove, BOOL bInsideClient);

public:
	virtual ~CZoomView();
#ifdef _DEBUG
	virtual void Dump(CDumpContext&) const;
	virtual void AssertValid() const;
#endif //_DEBUG
	virtual void CalcWindowRect(LPRECT lpClientRect,
		UINT nAdjustType = adjustBorder);
	virtual void OnPrepareDC(CDC* pDC, CPrintInfo* pInfo = NULL);

	// scrolling implementation support for OLE 2.0
	virtual BOOL OnScroll(UINT nScrollCode, UINT nPos, BOOL bDoScroll = TRUE);
	virtual BOOL OnScrollBy(CSize sizeScroll, BOOL bDoScroll = TRUE);

	//{{AFX_MSG(CZoomView)
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

// little helper class for zooming to a rectangle
// DON'T comment om this !!
class CZoomRect
{
public:
	BOOL TrackTheMouse( UINT uMessageEnd, CWnd *pWnd, CRect& rectZoom, CPoint ptAnchor );
protected:
	void RetrieveMsg( MSG* pmsg );
	BOOL ProcessMsg( MSG* pmsg, CWnd* pWnd );
	void UpdateRectangle( CWnd* pWnd, CPoint& pt );
	void DrawZoomRect( CDC* pDC );
	BOOL m_bFirstMove;
	BOOL m_bSuccess;
	UINT m_uMessageEnd;
	CPoint m_ptOrig;
	CPoint m_ptLast;
};
